# awk script to generate summary line for vCard entries
#
# See https://www.rfc-editor.org/rfc/rfc6350
#
# Limitations:
# - Use first FN, NICKNAME
# - Ignore properties of FN and NICKNAME

@include "lib/awk/vcard.awk"

BEGIN {
  # We require the following variables to be set using -v
  # collection_lables: ;-delimited collection=label strings
  # nickname_format: format string when nickname is present (has two placeholders)
  # fn_format: format string when nickname is absent (has one placeholders)
  # group_label: label for group entries
  # org_label: label for organizations
  # location_label: label for locations
  # individual_label: label for individuals

  FS = "[:;]";
  OFS = "\t"
  # Collections
  split(collection_labels, mapping, ";");
  for (map in mapping)
  {
    split(mapping[map], m, "=");
    collection2label[m[1]] = m[2];
  }
}

BEGINFILE {
  # Reset variables
  nickname = ""
  fn = ""
  kind = ""
  prop = ""
  line = ""
  delete c
}
FNR == 1 && /^BEGIN:VCARD/ {                                  next }
FNR == 1                   {                                  nextfile } # This is not a vcard file
/^END:VCARD/               {                                  nextfile } # Done, go to next file
/^(FN|NICKNAME)/ && c[$1]  { prop = "";                       next } # FN and NICKNAME may appear multiple times, take first
/^(FN|NICKNAME|KIND)/      { prop = $1; c[prop] = $0;         next }
/^[^ ]/ && prop            { prop = "";                       next }
/^ / && prop               { c[prop] = c[prop] substr($0, 2); next }
ENDFILE {
  # Construct path
  depth = split(FILENAME, path, "/")
  fpath = path[depth-1] "/" path[depth]
  # Collection name
  collection = path[depth-1]
  collection = collection in collection2label ? collection2label[collection] : collection
  # Content lines
  fn = singleline(unescape(getcontent(c["FN"])))
  nicknames = getcontent(c["NICKNAME"])
  if (nicknames) {
    split(nicknames, a, ",")
    for (i in a) {
      if (a[i] == fn)
        continue
      nickname = nickname " / " a[i]
    }
    nickname = substr(nickname, 4)
  }
  kind = getcontent(c["KIND"])
  switch(kind) {
    case "group":    kind = group_label;    break
    case "org":      kind = org_label;      break
    case "location": kind = location_label; break
    default:         kind = individual_label; # "individual"
  }
  
  # Build line to be presented
  line = nickname ? nickname_format : fn_format
  # If nickname contains the string "<<fn>>", then the behaviour is unexpected
  gsub("<<nickname>>", nickname, line)
  gsub("<<fn>>", fn, line)

  print line,
        collection,
        kind,
        fpath
}
