# Generate preview of vCard file
#
# Limitations:
# - Use first occurnece of FN, NICKNAME, ORG
# - other...
@include "lib/awk/vcard.awk"

# Returns type symbol fork "home" and "work" types.
# 
# @input type: type string
# @return: symbol or empty string
function type_symbol(type) {
  switch(type) {
    case "home": return label_home
    case "work": return label_work
    default: return ""
  }
}

# Returns phone symbol
# Empty string is mapped to the default "voice". Any other string not
# recognized resturns the empty string.
# 
# @input type: type string
# @return: nonempty string, defaults to voice symbol
function type_tel_symbol(type,    res) {
  res = type_symbol(type)
  if (res)
    return res
  switch(type) {
    case "text": return label_tel_text
    case "fax": return label_tel_fax
    case "cell": return label_tel_cell
    case "video": return label_tel_video
    case "pager": return label_tel_pager
    case "textphone": return label_tel_textphone
    case "voice": return label_tel
    case "": return label_tel
    default: return ""
  }
}

function preview_group() {
  print "GROUP"
}

function preview_org() {
  print "ORG"
}

function preview_location() {
  print "LOC"
}

function preview_individual() {
  # Parse unstructured name
  fn = singleline(unescape(getcontent(c["FN"])))
  nickname = getcontent(c["NICKNAME"])
  split(nickname, nicknamearray, ",")

  # Parse structured name
  if (c["N"]) {
    split(getcontent(c["N"]), a, ";")
    # Family name
    familyname = a[1]
    gsub(",", " ", familyname) # Comma-separated list, just concat with WS
    familyname = singleline(unescape(familyname))

    # Given name
    givenname = a[2]
    gsub(",", " ", givenname) # Comma-separated list, just concat with WS
    givenname = singleline(unescape(givenname))
    
    # Additional name
    additionalname = a[3]
    gsub(",", " ", additionalname) # Comma-separated list, just concat with WS
    additionalname = singleline(unescape(additionalname))
    
    # Prefix
    prefixname = a[4]
    gsub(",", " ", prefixname) # Comma-separated list, just concat with WS
    prefixname = singleline(unescape(prefixname))
    
    # Suffix
    suffixname = a[5]
    gsub(",", " ", suffixname) # Comma-separated list, just concat with WS
    suffixname = singleline(unescape(suffixname))

    # Print
    if (prefixname)
      line = line " " prefixname
    if (givenname)
      line = line " " givenname
    if (additionalname)
      line = line " " additionalname
    if (familyname)
      line = line " " familyname
    if (suffixname)
      line = line ? line ", " suffixname : suffixname
    line = substr(line, 2)
    fullname = line
    #if (line && line != fn && line != nickname)
    #  fullname = line
  }

  # Parse Tel
  tel[0] = 0
  delete tel[0]
  if (n["TEL"]) {
    k = 100
    for (i=1; i<= n["TEL"]; i++) {
      delete b
      tmp = ""
      nr = singleline(unescape(getcontent(c["TEL", i])))
      type = tolower(getparam(c["TEL", i], "TYPE"))
      gsub("\"", "", type)
      split(type, b, ",")
      for (j in b) {
        tmp = tmp type_tel_symbol(b[j])
      }
      type = tmp
      pref = tolower(getparam(c["TEL", i], "PREF"))
      if (!pref) {
        pref = k
        k = k + 1
      }
      while (pref in tel)
        pref = pref + 1
      tel[pref] = nr
      teltype[pref] = type
    }
  }

  # Parse E-Mail
  email[0] = 0
  delete email[0]
  if (n["EMAIL"]) {
    k = 100
    for (i=1; i<= n["EMAIL"]; i++) {
      delete b
      tmp = ""
      nr = singleline(unescape(getcontent(c["EMAIL", i])))
      type = tolower(getparam(c["EMAIL", i], "TYPE"))
      gsub("\"", "", type)
      split(type, b, ",")
      for (j in b) {
        tmp = tmp type_symbol(b[j])
      }
      type = tmp
      pref = tolower(getparam(c["EMAIL", i], "PREF"))
      if (!pref) {
        pref = k
        k = k + 1
      }
      while (pref in email)
        pref = pref + 1
      email[pref] = nr
      emailtype[pref] = type
    }
  }
  
  # Parse IMPP
  impp[0] = 0
  delete impp[0]
  if (n["IMPP"]) {
    k = 100
    for (i=1; i<= n["IMPP"]; i++) {
      delete b
      tmp = ""
      nr = singleline(unescape(getcontent(c["IMPP", i])))
      type = tolower(getparam(c["IMPP", i], "TYPE"))
      gsub("\"", "", type)
      split(type, b, ",")
      for (j in b) {
        tmp = tmp type_symbol(b[j])
      }
      type = tmp
      pref = tolower(getparam(c["IMPP", i], "PREF"))
      if (!pref) {
        pref = k
        k = k + 1
      }
      while (pref in impp)
        pref = pref + 1
      impp[pref] = nr
      impptype[pref] = type
    }
  }

  # Parse Address
  adr[0] = 0
  delete adr[0]
  if (n["ADR"]) {
    k = 100
    for (i=1; i<= n["ADR"]; i++) {
      delete b
      tmp = ""
      content = getcontent(c["ADR", i])
      split(content, aa, ";")
      pobox = aa[1]
      extad = aa[2]
      street = aa[3]
      locality = aa[4]
      region = aa[5]
      code = aa[6]
      country = aa[7]
      if (street)
        adrstr = singleline(unescape(street))
      if (locality) {
        if (adrstr)
          adrstr = adrstr "\n"
        adrstr = adrstr singleline(unescape(locality))
      }
      if (region) {
        if (!locality && adrstr)
          adrstr = adrstr "\n"
        adrstr = adrstr singleline(unescape(region))
      }
      if (code) {
        if (!locality && !region && adrstr)
          adrstr = adrstr "\n"
        adrstr = adrstr singleline(unescape(code))
      }
      if (country) {
        if (adrstr)
          adrstr = adrstr "\n"
        adrstr = adrstr singleline(unescape(country))
      }
      type = tolower(getparam(c["ADR", i], "TYPE"))
      gsub("\"", "", type)
      split(type, b, ",")
      for (j in b) {
        tmp = tmp type_symbol(b[j])
      }
      type = tmp
      pref = tolower(getparam(c["ADR", i], "PREF"))
      if (!pref) {
        pref = k
        k = k + 1
      }
      while (pref in adr)
        pref = pref + 1
      adr[pref] = adrstr
      adrtype[pref] = type
    }
  }

  # Parse birthday
  bday = getcontent(c["BDAY"])
  if (bday) {
    bday_date[0] = 0
    delete bday_date[0]
    date_and_or_time(bday, bday_date)
    omityear = getparam(c["BDAY"], "X-APPLE-OMIT-YEAR")
    if (omityear == bday_date["year"])
      bday_date["year"] = ""
  }


  # Print structured data
  if (fullname)
    print "# "fullname
  # Phone numbers
  if (length(tel) >= 1) {
    print "\n## ☎️ Phone numbers"
    for (pref in tel) {
      style = pref == 1 ? "**" : ""
      print "- " style teltype[pref] "\t" tel[pref] style
    }
  }
  # E-Mail addresses
  if (length(email) >= 1) {
    print "\n## 📧 E-mail addresses"
    for (pref in email) {
      style = pref == 1 ? "**" : ""
      print "- " style emailtype[pref] "\t" email[pref] style
    }
  }
  # IMPP
  if (length(impp) >= 1) {
    print "\n## 💬 Instant messaging"
    for (pref in impp) {
      style = pref == 1 ? "**" : ""
      print "- " style impptype[pref] "\t" impp[pref] style
    }
  }
  # Address
  if (length(adr) >= 1) {
    for (pref in adr) {
      print "\n## " adrtype[pref] " Postal address"
      print adr[pref]
    }
  }
  # Birthday
  if (bday_date["month"] && bday_date["day"]) {
    datestr = "2000-"bday_date["month"] "-" bday_date["day"]
    gsub("\"", "", datestr)
    cmd = "date -d \"" datestr "\" +\"%e %B\""
    suc = cmd | getline res
    close(cmd)
    if (suc == 1) {
      print "\n### 🎂 Birthday"
      if (bday_date["year"])
        print res " " bday_date["year"]
      else
        print res
    }
  }
  # Nicknames
  if (nickname) {
    print "\n### 🏷️ Additional names"
    for (i in nicknamearray)
      print "- " nicknamearray[i]
  }
}

BEGIN                            { IGNORECASE=1; FS = "[:;]" }
FNR == 1 && /^BEGIN:VCARD/       {                                  next }
FNR == 1                         {                                  exit 1 } # This is not a vcard file
/^END:VCARD/                     { exit }
/^(FN|NICKNAME|ORG)/ && c[$1]    { prop = "";                next } # FN, NICKNAME, ORG may appear multiple times, take first
/^(FN|NICKNAME|KIND|N|BDAY|ORG)/ { prop = toupper($1); n[prop] = 0; c[prop] = $0; next }
/^(ADR|TEL|EMAIL|IMPP|GEO|TITLE|ROLE|ORG|URL)/ { # These entries may appear multiple times
                                   prop = toupper($1); n[prop] = n[prop] + 1; c[prop, n[prop]] = $0; next }
/^[^ ]/ && prop                  { prop = "";                       next }
/^ / && prop {
  if (n[prop])
    c[prop, n[prop]] = c[prop, n[prop]] substr($0, 2)
  else
    c[prop] = c[prop] substr($0, 2)
  next
}
END {
  kind = getcontent(c["KIND"])
  switch(kind) {
    case "group":    preview_group();      break
    case "org":      preview_org();        break
    case "location": preview_location();   break
    default:         preview_individual(); break
  }
}
