# Get date from date-and-or-time value
#
# @input str: date-and-or-time string
# @input arr: array, where arr["year"] arr["month"] arr["day"] will be the
#             parsed date
function date_and_or_time(str, arr) {
  # Some applications hyphenate the string.
  # Year and month portions can be omitted by putting dashes.
  if (index(str, "T"))
    str = substr(str, 1, index(str, "T") - 1)
  switch(length((str))) {
    case 8:
      arr["year"] = substr(str, 1, 4)
      arr["month"] = substr(str, 5, 2)
      arr["day"] = substr(str, 7, 2)
      return
    case 10:
      arr["year"] = substr(str, 1, 4)
      arr["month"] = substr(str, 6, 2)
      arr["day"] = substr(str, 9, 2)
      return
    case 4:
      arr["year"] = substr(str, 1, 4)
      arr["month"] = ""
      arr["day"] = ""
      return
    case 7:
      arr["year"] = substr(str, 1, 4)
      arr["month"] = substr(str, 6, 2)
      arr["day"] = ""
      return
    case 6:
      arr["year"] = ""
      arr["month"] = substr(str, 3, 2)
      arr["day"] = substr(str, 5, 2)
      return
    case 5:
      arr["year"] = ""
      arr["month"] = ""
      arr["day"] = substr(str, 4, 2)
      return
  }
  arr["year"] = ""
  arr["month"] = ""
  arr["day"] = ""
  return
}

# Make string single-line
#
# @input str: String
# @return: String without newlines
function singleline(str) {
  gsub("\\n", " ", str)
  return str
}

# Escape string to be used as content in iCalendar files.
#
# @input str: String to escape
# @return: Escaped string
function escape(str)
{
  gsub("\\\\", "\\\\", str)
  gsub("\\n",  "\\n",  str)
  gsub(";",    "\\;",  str)
  gsub(",",    "\\,",  str)
  return str
}

# Escape string to be used as content in iCalendar files, but don't escape
# commas.
#
# @input str: String to escape
# @return: Escaped string
function escape_but_commas(str)
{
  gsub("\\\\", "\\\\", str)
  gsub("\\n",  "\\n",  str)
  gsub(";",    "\\;",  str)
  return str
}

# Print property with its content and fold according to the iCalendar
# specification.
#
# @local variables: i, s
# @input nameparam: Property name with optional parameters
# @input content: Escaped content
function print_fold(nameparam, content,    i, s)
{
  i = 74 - length(nameparam)
  s = substr(content, 1, i)
  print nameparam s
  s = substr(content, i+1, 73)
  i = i + 73
  while (s)
  {
    print " " s
    s = substr(content, i+1, 73)
    i = i + 73
  }
}

# Unescape string
#
# @local variables: i, c, c2, res
# @input str: String
# @return: Unescaped string
function unescape(str,    i, c, c2, res) {
  for(i = 1; i <= length(str); i++) {
    c = substr(str, i, 1)
    if (c != "\\") {
      res = res c
      continue
    }
    i++
    c2 = substr(str, i, 1)
    if (c2 == "n" || c2 == "N") {
      res = res "\n"
      continue
    }
    # Alternatively, c2 is "\\" or "," or ";". In each case, append res with
    # c2. If the strings has been escaped correctly, then the character c2
    # cannot be anything else. To be fail-safe, simply append res with c2.
    res = res c2
  }
  return res
}

# Get specific parameter of logical line.
# Parameter names are case insentitive.
#
# @input str: logical line
# @input param: Parameter name
# @return: Parameter value, comma separated
function getparam(str, param,    a, i, paramname, res) {
  i = index(str, ";")
  if (!i)
    return ""
  str = substr(str, i + 1)
  str = substr(str, 1, index(str, ":") - 1)
  split(str, a, ";")
  for (i in a) {
    paramname = substr(a[i], 1, index(a[i], "=") - 1)
    if (toupper(paramname) == toupper(param))
      res = res "," substr(a[i], index(a[i], "=") + 1)
  }
  res = substr(res, 2)
  return res
}

# Isolate content part of an iCalendar line, and unescape.
#
# @input str: String
# @return: Escaped content part
function getcontent(str) {
  return substr(str, index(str, ":") + 1)
}
